; -----------------------------------------
; THE improved 'NATURAL LOGARITHM' FUNCTION
; -----------------------------------------
; (offset $22: 'ln')
;   Like the ZX81 itself, 'natural' logarithms came from Scotland.
;   They were devised in 1614 by well-traveled Scotsman John Napier who noted
;   "Nothing doth more molest and hinder calculators than the multiplications,
;    divisions, square and cubical extractions of great numbers".
;   Napier's logarithms enabled the above operations to be accomplished by 
;   simple addition and subtraction simplifying the navigational and 
;   astronomical calculations which beset his age.
;   Napier's logarithms were quickly overtaken by logarithms to the base 10
;   devised, in conjunction with Napier, by Henry Briggs a Cambridge-educated 
;   professor of Geometry at Oxford University. These simplified the layout
;   of the tables enabling humans to easily scale calculations.
;
;   It is only recently with the introduction of pocket calculators and
;   computers like the ZX81 that natural logarithms are once more at the fore,
;   although some computers retain logarithms to the base ten.
;   'Natural' logarithms are powers to the base 'e', which like 'pi' is a 
;   naturally occurring number in branches of mathematics.
;   Like 'pi' also, 'e' is an irrational number and starts 2.718281828...
;
;   The tabular use of logarithms was that to multiply two numbers one looked
;   up their two logarithms in the tables, added them together and then looked 
;   for the result in a table of antilogarithms to give the desired product.
;
;   The EXP function is the BASIC equivalent of a calculator's 'antiln' function 
;   and by picking any two numbers, 1.72 and 6.89 say,
;	10 PRINT EXP ( LN 1.72 + LN 6.89 ) 
;   will give just the same result as
;	20 PRINT 1.72 * 6.89.
;   Division is accomplished by subtracting the two logs.
;
;   Napier also mentioned "square and cubicle extractions". 
;   To raise a number to the power 3, find its 'ln', multiply by 3 and find the 
;   'antiln'.  e.g. PRINT EXP( LN 4 * 3 )  gives 64.
;   Similarly to find the n'th root divide the logarithm by 'n'.
;
;   First test that the argument to LN is a positive, non-zero number.
fn_ln
;;-	RST 28H		;; FP-CALC		x.
;;-	DEFB $2D	;;duplicate		x,x.

;;-	DEFB $33	;;greater-0		x,(0/1).
;;-	DEFB $00	;;jump-true		x.
;;-	DEFB VALID-$	;;to VALID

;;-	DEFB $34	;;end-calc		x.
;;-
;;- REPORT_Ab
;;-	RST 08H			; ERROR-1
;;-	DEFB $09		; Error Report: Invalid argument
;;
;;- VALID  
;;-	DEFB $34	;;end-calc		x.
;
;   Register HL addresses the 'last value' x.
;
;;-	LD A,(HL)		; Fetch exponent to A.

	inc hl			; Address byte with sign bit.
	bit 7,(hl)		; Test the bit. 
	jr nz,REPORT_A		; if <0 then REPORT_A: 'Invalid argument'

	dec hl			; HL now points to exponent
	ld a,(hl)		; Fetch exponent to A.
	and a			; Test for zero argument
	jr z,REPORT_A		; if =0 then REPORT_A: 'Invalid argument'
;
	LD (HL),$80		; Insert 'plus zero' as exponent.
	CALL STACK_A		; routine STACK-A stacks true binary exponent.
		
	RST 28H		;; FP-CALC
	DEFB $30	;;stk-data
	DEFB $38	;;Exponent: $88, Bytes: 1
	DEFB $00	;;(+00,+00,+00)
	DEFB $03	;;subtract
	DEFB $01	;;exchange
	DEFB $2D	;;duplicate
	DEFB $30	;;stk-data
	DEFB $F0	;;Exponent: $80, Bytes: 4
	DEFB $4C,$CC,$CC,$CD ;;
	DEFB $03	;;subtract

	DEFB $33	;;greater-0
	DEFB $00	;;jump-true
	DEFB GRE_8-$	;;to GRE.8

	DEFB $01	;;exchange

;;-	DEFB $A1	;;stk-one
;;-	DEFB $03	;;subtract

	.db $39		;;sub-one macro

	DEFB $01	;;exchange

;;-	DEFB $34	;;end-calc
;;-	INC (HL)		; *2
;;-	RST   28H	;; FP-CALC

	.db $3A		;;mul-by-2			*2
GRE_8
	DEFB $01	;;exchange
	DEFB $30	;;stk-data			LN 2
	DEFB $F0	;;Exponent: $80, Bytes: 4
	DEFB $31,$72,$17,$F8 ;;
	DEFB $04	;;multiply
	DEFB $01	;;exchange

;;-	DEFB $A2	;;stk-half
;;-	DEFB $03	;;subtract
;;-	DEFB $A2	;;stk-half
;;-	DEFB $03	;;subtract

	.db $39		 ;;sub-one macro

	DEFB $2D	;;duplicate
	DEFB $30	;;stk-data
	DEFB $32	;;Exponent: $82, Bytes: 1
	DEFB $20	;;(+00,+00,+00)
	DEFB $04	;;multiply
	DEFB $A2	;;stk-half
	DEFB $03	;;subtract
	DEFB $8C	;;series-0C
	DEFB $11	;;Exponent: $61, Bytes: 1
	DEFB $AC	;;(+00,+00,+00)
	DEFB $14	;;Exponent: $64, Bytes: 1
	DEFB $09	;;(+00,+00,+00)
	DEFB $56	;;Exponent: $66, Bytes: 2
	DEFB $DA,$A5	;;(+00,+00)
	DEFB $59	;;Exponent: $69, Bytes: 2
	DEFB $30,$C5	;;(+00,+00)
	DEFB $5C	;;Exponent: $6C, Bytes: 2
	DEFB $90,$AA	;;(+00,+00)
	DEFB $9E	;;Exponent: $6E, Bytes: 3
	DEFB $70,$6F,$61 ;;(+00)
	DEFB $A1	;;Exponent: $71, Bytes: 3
	DEFB $CB,$DA,$96 ;;(+00)
	DEFB $A4	;;Exponent: $74, Bytes: 3
	DEFB $31,$9F,$B4 ;;(+00)
	DEFB $E7	;;Exponent: $77, Bytes: 4
	DEFB $A0,$FE,$5C,$FC ;;
	DEFB $EA	;;Exponent: $7A, Bytes: 4
	DEFB $1B,$43,$CA,$36 ;;
	DEFB $ED	;;Exponent: $7D, Bytes: 4
	DEFB $A7,$9C,$7E,$5E ;;
	DEFB $F0	;;Exponent: $80, Bytes: 4
	DEFB $6E,$23,$80,$93 ;;
	DEFB $04	;;multiply
	DEFB $0F	;;addition
	DEFB $34	;;end-calc

	RET			; return.
;
; ------------
; THE REPORT_A
; ------------
;
REPORT_A

	RST 08H			; ERROR-1
	DEFB $09		; Error Report: Invalid argument
