; -------------------------
; THE old 'COSINE' FUNCTION
; -------------------------
; (offset $1D: 'cos')
;;-  Cosines are calculated as the sine of the opposite angle rectifying the 
;;-  sign depending on the quadrant rules. 
;;-
;;-	    /|
;;-	 h /y|
;;-	  /  |o
;;-	 /x  |
;;-	/----|    
;;-	  a
;;-
;;-  The cosine of angle x is the adjacent side (a) divided by the hypotenuse 1.
;;-  However if we examine angle y then a/h is the sine of that angle.
;;-  Since angle x plus angle y equals a right-angle, we can find angle y by 
;;-  subtracting angle x from pi/2.
;;-  However it's just as easy to reduce the argument first and subtract the
;;-  reduced argument from the value 1 (a reduced right-angle).
;;-  It's even easier to subtract 1 from the angle and rectify the sign.
;;-  In fact, after reducing the argument, the absolute value of the argument
;;-  is used and rectified using the test result stored in mem-0 by 'get-argt'
;;-  for that purpose.
;;-
;;-cos
;;-	RST 28H		;; FP-CALC		angle in radians.
;;-	DEFB $35	;;get-argt		X  reduce -1 to +1

;;-	DEFB $27	;;abs			ABS X   0 to 1
;;-	DEFB $A1	;;stk-one		ABS X, 1.
;;-	DEFB $03	;;subtract		now opposite angle 
			;;		 	though negative sign.
;;-	DEFB $E0	;;get-mem-0		fetch sign indicator.
;;-	DEFB $00	;;jump-true
;;-	DEFB $06	;;fwd to L1D4B, C-ENT
			;;forward to common code if in QII or QIII 

;;-	DEFB $18	;;negate		else make positive.
;;-	DEFB $2F	;;jump
;;-	DEFB $03	;;fwd to L1D4B, C-ENT
			;;with quadrants QI and QIV 

;
; -------------------------
; THE new 'COSINE' FUNCTION
; -------------------------
; (offset $1D: 'cos')
;
; Cosines are calculated as COS(X)=SIN(X + PI/2)
;
fn_cos
	RST 28H		;; FP-CALC		angle in radians
	.db $A2		;;stk-half  		X, 0.5 (offset)

	.db $2F		;;jump
	.db cos_entr-$	;;to cos_entr		continue as SINE function
;
; ----------------------------
; THE improved 'SINE' FUNCTION
; ----------------------------
; (offset $1C: 'sin')
;   This is a fundamental transcendental function from which others such as cos
;   and tan are directly, or indirectly, derived.
;   It uses the series generator to produce Chebyshev polynomials.
;
;	    /|
;	 1 / |
;	  /  |x
;	 /a  |
;	/----|    
;	  y
;
;   The 'get-argt' part is designed to modify the angle and its sign 
;   in line with the desired sine value.
;
;;- sin
;;-	RST 28H		;; FP-CALC		angle in radians

;;-	DEFB $35	;;get-argt		reduce - sign now correct.
;;-C_ENT

; ------------------------------
; THE improved 'REDUCE ARGUMENT'
; ------------------------------
;
;   This part performs two functions on the angle, in radians, that forms
;   the argument to the sine (and cosine) functions.
;   First it ensures that the angle 'wraps round'. That if a ship turns through 
;   an angle of, say, 3*PI radians (540 degrees) then the net effect is to turn 
;   through an angle of PI radians (180 degrees).
;   Secondly it converts the angle in radians to a fraction of a right angle,
;   depending within which quadrant the angle lies, with the periodicity 
;   resembling that of the desired sine value.
;   The result lies in the range -1 to +1.	
;
;                       90 deg.
; 
;                       (pi/2)
;                II       +1        I
;                         |
;          sin+      |\   |   /|    sin+
;          cos-      | \  |  / |    cos+
;          tan-      |  \ | /  |    tan+
;                    |   \|/)  |           
;   180 deg. (pi) 0 -|----+----|-- 0  (0)   0 degrees
;                    |   /|\   |
;          sin-      |  / | \  |    sin-
;          cos-      | /  |  \ |    cos+
;          tan+      |/   |   \|    tan-
;                         |
;                III      -1       IV
;                       (3pi/2)
;
;                       270 deg.
;
fn_sin
	RST 28H		;; FP-CALC		angle in radians

	.db $A0		;;stk-zero  		X, 0.	 (offset)
cos_entr
	.db $01		;;exchange		ofs, X.

	DEFB $30	;;stk-data
	DEFB $EE	;;Exponent: $7E, Bytes: 4
	DEFB $22,$F9,$83,$6E ;;			X, 1/(2*PI)  
	DEFB $04	;;multiply		X/(2*PI) = fraction

	DEFB $2D	;;duplicate  		fraction, fraction
	DEFB $A2	;;stk-half  		fraction, fraction, 0.5
	DEFB $0F	;;addition  		fraction, fraction + 0.5
	DEFB $24	;;int  			fraction, int(fraction+0.5)

	DEFB $03	;;subtract		now range -.5 to .5

;;-	DEFB $2D	;;duplicate
;;-	DEFB $0F	;;addition		now range -1 to 1.

	.db $3A		;;mul-by-2		ofs, now range -1 to 1.
	.db $0F		;;addition 		ofs + range (-1 to 1).

	.db $2D		;;duplicate		ofs_rng, ofs_rng
	.db $39		;;sub-one macro		ofs_rng, ofs_rng-1.

	.db $2D		;;duplicate		ofs_rng, ofs_rng-1, ofs_rng-1.
	.db $32		;;less-0		ofs_rng, ofs_rng-1, 0/1.
	.db $00		;;jump-true
	.db no_qchg-$	;;to no_qchg 		

	.db $39		;;sub-one macro		ofs_rng, ofs_rng-2.
	.db $01		;;exchange		ofs_rng-2, ofs_rng.
no_qchg
	.db $02		;;delete		delete test value.

;;-	DEFB $2D	;;duplicate
;;-	DEFB $0F	;;addition		now range -2 to 2.

	.db $3A		;;mul-by-2		now range -2 to 2.
;
;   quadrant I (0 to +1) and quadrant IV (-1 to 0) are now correct.
;   quadrant II ranges +1 to +2.
;   quadrant III ranges -2 to -1.
;
	DEFB $2D	;;duplicate		Y, Y.
	DEFB $27	;;abs			Y, abs(Y).    range 1 to 2

;;-	DEFB $A1	;;stk-one		Y, abs(Y), 1.
;;-	DEFB $03	;;subtract		Y, abs(Y)-1.  range 0 to 1

	.db $39		;;sub-one macro		Y, abs(Y)-1.  range 0 to 1

	DEFB $2D	;;duplicate		Y, Z, Z.

	DEFB $33	;;greater-0		Y, Z, (1/0).
;;-	DEFB $C0	;;st-mem-0	store as possible sign 
;;-			;;		for cosine function.
	DEFB $00	;;jump-true
	DEFB ZPLUS-$	;;to ZPLUS	with quadrants II and III
;
;   else the angle lies in quadrant I or IV and value Y is already correct.
;
	DEFB $02	;;delete		Y	delete test value.
;;-	DEFB $34	;;end-calc		Y.

;;-	RET			; return. with Q1 and Q4 >>>

	.db $2F		;;jump
	.db YNEG-$	;;to YNEG		Y.	with Q1 and Q4 >>>
;
;   The branch was here with quadrants II (0 to 1) and III (1 to 0).
;   Y will hold -2 to -1 if this is quadrant III.
;
ZPLUS
;;-	DEFB $A1	;;stk-one		Y, Z, 1
;;-	DEFB $03	;;subtract		Y, Z-1.  Q3 = 0 to -1

	.db $39		;;sub-one macro		Y, Z-1.  Q3 = 0 to -1

	DEFB $01	;;exchange		Z-1, Y.

	DEFB $32	;;less-0		Z-1, (1/0).
	DEFB $00	;;jump-true		Z-1.
	DEFB YNEG-$	;;to YNEG		if angle in quadrant III
;
;   else angle is within quadrant II (-1 to 0)
;
	DEFB $18	;;negate		range +1 to 0
;
;	-------------------------------------------
YNEG
;;-	DEFB $34	;;end-calc	quadrants II and III correct.

;;-	RET			; return.

;;-	DEFB $2D	;;duplicate		x, x.
;;-	DEFB $2D	;;duplicate		x, x, x.
;;-	DEFB $04	;;multiply		x, x*x.

	.db $3C		;;stk-square		x, x*x.

;;-	DEFB $2D	;;duplicate		x, x*x, x*x.
;;-	DEFB $0F	;;addition		x, 2*x*x.

	.db $3A		;;mul-by-2		x, 2*x*x.

;;-	DEFB $A1	;;stk-one		x, 2*x*x, 1.
;;-	DEFB $03	;;subtract		x, 2*x*x-1.

	.db $39		;;sub-one macro		x, 2*x*x-1

	DEFB $86	;;series-06
	DEFB $14	;;Exponent: $64, Bytes: 1
	DEFB $E6	;;(+00,+00,+00)
	DEFB $5C	;;Exponent: $6C, Bytes: 2
	DEFB $1F,$0B	;;(+00,+00)
	DEFB $A3	;;Exponent: $73, Bytes: 3
	DEFB $8F,$38,$EE ;;(+00)
	DEFB $E9	;;Exponent: $79, Bytes: 4
	DEFB $15,$63,$BB,$23 ;;
	DEFB $EE	;;Exponent: $7E, Bytes: 4
	DEFB $92,$0D,$CD,$ED ;;
	DEFB $F1	;;Exponent: $81, Bytes: 4
	DEFB $23,$5D,$1B,$EA ;;

	DEFB $04	;;multiply		x*series_06
	DEFB $34	;;end-calc
;
; -------------------------------------------
; THE eliminated 'REDUCE ARGUMENT' SUBROUTINE
; -------------------------------------------
; (offset $35: 'get-argt')
;
; now it is part of the sine function - see above
;
get_argt
	RET 	; return.
